/* spi.c */

// Port mit den SPI-Pins:
#define SPI_DDR DDRB
#define SPI_SCK  1 //SCK
#define SPI_MOSI 2 //MOSI
#define SPI_MISO 3 //MISO

// Port zum Geraet selektieren:
#ifndef SPI_SEL_PORT
#define SPI_SEL_PORT PORTB
#define SPI_SEL_DDR  DDRB
#define SPI_SEL_BIT  0
#endif

//------------------------------------------------------------------------------------------
void spi_init() //SPI initialisieren, aehnlich SPI_MasterInit() im Datenblatt vom Atmega2560
{
 SPI_SEL_PORT |= (1<<SPI_SEL_BIT); //SEL auf High
 SPI_SEL_DDR  |= (1<<SPI_SEL_BIT); //SEL als Ausgang

 SPI_DDR |= (1<<SPI_SCK);  //SCK als Ausgang
 SPI_DDR |= (1<<SPI_MOSI); //MOSI als Ausgang
 SPI_DDR &= ~(1<<SPI_MISO); //MISO als Eingang

 //SPSR = (1<<SPI2X); //Geschwindigkeit verdoppeln (auskommentieren fuer normale Geschwindigkeit)

// genau eine der folgenden 4 Zeilen nicht auskommentieren:
 SPCR = (1<<SPE) | (1<<MSTR) | (1<<SPR1)|(1<<SPR0); //kleinste Geschwindigkeit, Teiler=128
 //SPCR = (1<<SPE) | (1<<MSTR) | (1<<SPR1); //Teiler=64
 //SPCR = (1<<SPE) | (1<<MSTR) | (1<<SPR0); //Teiler=16
 //SPCR = (1<<SPE) | (1<<MSTR);             //groesste Geschwindigkeit, Teiler=4

 //SPCR |= (1<<DORD); //LSB first
 //SPCR |= (1<<CPOL); //SCK high when idle
 //SPCR |= (1<<CPHA); //Daten mit zweiter SCK-Flanke uebernehmen

 //SPCR |= (1<<SPIE); //Interrupt einschalten
}

void spi_send(char c) //gleich wie SPI_MasterTransmit() im Datenblatt vom Atmega2560
{
 SPDR = c; //Starte Uebertragung
 while(!(SPSR & (1<<SPIF)))
   ; //warte bis Uebertragung fertig
}

unsigned char spi_transmit(unsigned char c) //Senden und Empfangen gleichzeitig
{
 SPDR = c; //Starte Uebertragung
 while(!(SPSR & (1<<SPIF)))
   ; //warte bis Uebertragung fertig
 return SPDR;
}
